import argparse
import logging
import sys
from typing import Dict
from typing import Optional
from typing import Sequence
from typing import Tuple

from silx.gui import qt

from ..gui.settings import clear_settings
from ..gui.xasmap_viewer import XasMapViewer

_logger = logging.getLogger(__name__)


def _absorb_nonbase_exception(exc_type, exc_value, exc_traceback) -> None:
    if not issubclass(exc_type, Exception):
        sys.__excepthook__(exc_type, exc_value, exc_traceback)
        return

    _logger.error("Uncaught exception", exc_info=(exc_type, exc_value, exc_traceback))


def _cli(
    args: Optional[Sequence[str]] = None,
) -> Tuple[Dict[str, Optional[str]], bool, int]:
    parser = argparse.ArgumentParser(description="XAS Map Viewer")
    parser.add_argument(
        "filenames",
        nargs="*",
        help="Zero or more data files to load in the XAS Map Viewer",
    )
    parser.add_argument(
        "--dim0-counter",
        default=None,
        help="X-axis channel name",
    )
    parser.add_argument(
        "--dim1-counter",
        default=None,
        help="Y-axis channel name",
    )
    parser.add_argument(
        "--energy-counter",
        default=None,
        help="Primary beam energy channel name",
    )
    parser.add_argument(
        "--mu-counter",
        default=None,
        help="Mu channel name",
    )
    parser.add_argument(
        "--scan",
        default=None,
        type=str,
        help="Scan number",
    )
    parser.add_argument("--fresh", action="store_true")
    parser.add_argument(
        "-v",
        "--verbose",
        action="count",
        default=0,
        help="Increase verbosity",
    )

    values = parser.parse_args(args=args)
    select_inputs = {
        "filenames": values.filenames,
        "dim0_counter": values.dim0_counter,
        "dim1_counter": values.dim1_counter,
        "energy_counter": values.energy_counter,
        "mu_counter": values.mu_counter,
        "scan_ranges": values.scan,
    }

    return select_inputs, values.fresh, values.verbose


def main(args: Optional[Sequence[str]] = None):
    logging.basicConfig(level=logging.WARNING)
    sys.excepthook = _absorb_nonbase_exception
    select_inputs, fresh, verbosity = _cli(args=args)

    if verbosity == 1:
        logging.getLogger("ewoksid24").setLevel(logging.INFO)
    elif verbosity == 2:
        logging.getLogger().setLevel(logging.INFO)
        logging.getLogger("ewoksid24").setLevel(logging.DEBUG)
    elif verbosity >= 3:
        logging.getLogger().setLevel(logging.DEBUG)

    app = qt.QApplication([])
    window = XasMapViewer()

    if fresh:
        clear_settings()

    try:
        window.load_settings()
    except Exception:
        _logger.exception("Failed loading the Qt settings")

    window.add_xasmap(**select_inputs)

    window.show()
    sys.exit(app.exec_())


if __name__ == "__main__":
    main()



import logging
from typing import Optional

import numpy
import scipy.constants
from scipy.optimize import curve_fit

from ..io.temperature import TemperatureData

_logger = logging.getLogger(__name__)


def spectral_radiance(wavelength: numpy.ndarray, temperature: float, scale: float):
    r"""Planck's law describing the spectral density of electromagnetic radiation emitted by
    a black body in thermal equilibrium at a given temperature T.

    .. math::

        B(\lambda, T) = \frac{2hc^2}{\lambda^5} \cdot \frac{1}{e^{\frac{hc}{\lambda kT} - 1}}

    Where:

    - :math:`B(\lambda, T)` is the spectral radiance (:math:`\frac{W}{m^3\text{sr}}`) at wavelength :math:`\lambda` and temperature :math:`T`.
    - :math:`h` is Planck's constant (:math:`6.62607015 \times 10^{-34}` J·s).
    - :math:`c` is the speed of light in vacuum (:math:`2.998 \times 10^8` m/s).
    - :math:`k` is the Boltzmann constant (:math:`1.380649 \times 10^{-23}` J/K).
    - :math:`\lambda` is the wavelength of the radiation in meters.
    - :math:`T` is the temperature of the blackbody radiator in Kelvin.

    :param wavelength: wavelength in nanometer
    :param temperature: temperature in Kelvin
    :param scale: scaling factor to fit the data
    :returns: spectral radiance
    """
    h = scipy.constants.Planck  # J·s
    c = scipy.constants.speed_of_light  # m/s
    k = scipy.constants.Boltzmann  # J/K

    lam = wavelength * 1e-9
    exp = numpy.exp((h * c) / (lam * k * temperature))
    radiance = (2 * h * c**2) / (lam**5) / (exp - 1)
    return radiance * scale


def fit_temperature_data(
    temp_data: TemperatureData,
    wavelength_min: Optional[float] = None,
    wavelength_max: Optional[float] = None,
) -> None:
    """Fit temperature data and overwrite the existing fit results."""
    yfit = numpy.zeros(temp_data.planck_fit.shape[1], dtype=temp_data.planck_fit.dtype)

    xyiter = zip(temp_data.wavelength, temp_data.planck_data)
    for i, (x, y) in enumerate(xyiter):
        if wavelength_min is None:
            start = temp_data.planck_fit_slice.start
        else:
            start = numpy.argmin(numpy.abs(x - wavelength_min))
        if wavelength_max is None:
            stop = temp_data.planck_fit_slice.stop
        else:
            stop = numpy.argmin(numpy.abs(x - wavelength_max)) + 1
        fit_slice = slice(start, stop)
        temp_data.planck_fit_slice = fit_slice
        x = x[fit_slice]
        y = y[fit_slice]

        temperature0 = temp_data.planck_temperature[i]
        scale0 = numpy.median(y / spectral_radiance(x, temperature0, 1))
        p0 = [temperature0, scale0]

        (temperature, scale), _ = curve_fit(spectral_radiance, x, y, p0=p0)

        yfit[fit_slice] = spectral_radiance(x, temperature, scale)
        temp_data.planck_fit[i] = yfit

        _logger.debug(
            "Point %d: %f K -> %f K", i, temp_data.planck_temperature[i], temperature
        )
        temp_data.planck_temperature[i] = temperature



from copy import deepcopy

from ewokscore import Task

from ..math.planck import fit_temperature_data


class PlanckRadianceFit(
    Task,
    input_names=["temp_data"],
    optional_input_names=["wavelength_min", "wavelength_max"],
    output_names=["temp_data"],
):
    """Fit Black Body radiance with Planck's law."""

    def run(self):
        temp_data = deepcopy(self.inputs.temp_data)
        wavelength_min = self.get_input_value("wavelength_min", None)
        wavelength_max = self.get_input_value("wavelength_max", None)
        fit_temperature_data(
            temp_data, wavelength_min=wavelength_min, wavelength_max=wavelength_max
        )
        self.outputs.temp_data = temp_data



import os
import sys

import matplotlib


def init_matplotlib():
    if _is_headless():
        # For the plot tasks in headless execution
        matplotlib.use("agg")


def _is_headless() -> bool:
    try:
        return not os.isatty(sys.stdout.fileno())
    except AttributeError:
        # celery worker subprocess patches sys.stdout
        return True



from ..io.xasmap import XasMapData
from ..io.xasmap import read_xasmap
from ..math.xasmap import XasMapInterpolator


def test_xasmap_id24_dcm(id24_dcm_xasmap):
    xasmap = read_xasmap(**id24_dcm_xasmap)
    _assert_xasmap(xasmap)


def test_xasmap_id24_ed(id24_ed_xasmap):
    xasmap = read_xasmap(**id24_ed_xasmap)
    _assert_xasmap(xasmap)


def _assert_xasmap(xasmap: XasMapData):
    assert xasmap.energy_units == "keV"
    assert xasmap.x0_units == "um"
    assert xasmap.x1_units == "um"


def test_xasmap_interpolator(id24_dcm_xasmap):
    xasmap = read_xasmap(**id24_dcm_xasmap)
    interpolator = XasMapInterpolator(xasmap)
    assert interpolator.energy_label == "Energy (keV)"
    assert interpolator.x0_axis_label == "x (um)"
    assert interpolator.x1_axis_label == "y (um)"



import os

from silx.gui import qt

from ...gui.xasmap_select import XasMapInputWidget


def test_add_xasmap(qtbot):
    fake_files = ["/tmp/file1.h5", "/tmp/file2.h5"]
    widget = XasMapInputWidget()
    widget.add_xasmap(
        filenames=fake_files,
        dim0_counter="x",
        dim1_counter="y",
        energy_counter="energy",
        mu_counter="mu",
        scan_ranges="[[1, 5]]",
        exclude_scans="[[2, 3]]",
    )
    qtbot.addWidget(widget)

    xasmaps = widget._get_xasmaps()
    assert len(xasmaps) == 1

    xasmap = xasmaps[0]
    assert xasmap["filenames"] == list(map(os.path.realpath, fake_files))
    assert xasmap["dim0_counter"] == "x"
    assert xasmap["dim1_counter"] == "y"
    assert xasmap["energy_counter"] == "energy"
    assert xasmap["mu_counter"] == "mu"
    assert xasmap["scan_ranges"] == [[1, 5]]
    assert xasmap["exclude_scans"] == [[2, 3]]


def test_invalid_scan_range_warns(monkeypatch):
    widget = XasMapInputWidget()
    widget.add_xasmap(
        filenames=["/tmp/file1.h5"],
        dim0_counter="x",
        dim1_counter="y",
        energy_counter="energy",
        mu_counter="mu",
    )
    assert len(widget._get_xasmaps()) == 1

    widget._scan_range_input.setText("[[1]]")  # Invalid scan range (only one value)

    called = False

    def fake_warning(*args, **kwargs):
        nonlocal called
        called = True

    monkeypatch.setattr(qt.QMessageBox, "warning", fake_warning)

    widget._create_xasmap()

    assert called is True
    assert len(widget._get_xasmaps()) == 1



import logging
from concurrent.futures import CancelledError
from concurrent.futures import Future
from concurrent.futures import ThreadPoolExecutor
from typing import Callable
from typing import Dict
from typing import Optional

import numpy
from silx.gui import icons
from silx.gui import qt
from silx.gui.colors import Colormap
from silx.gui.plot import Plot1D
from silx.gui.plot import Plot2D
from silx.gui.plot.CurvesROIWidget import ROI
from silx.gui.utils import blockSignals
from silx.gui.utils.concurrent import submitToQtMainThread

from ..io.xasmap import XasMapData
from ..io.xasmap import read_xasmap
from ..math.xasmap import XasMapInterpolator
from .settings import get_settings
from .xasmap_select import XasMapInputWidget

_logger = logging.getLogger(__name__)


class XasMapViewer(qt.QMainWindow):
    plot_updated = (
        qt.Signal()
    )  # Emits when data loading and plotting has completed (success or failure)

    def __init__(self, parent=None):
        super().__init__(parent=parent)

        self.setWindowTitle("XAS Map Viewer")

        self._settings: Optional[qt.QSettings] = None

        # XAS map selecting
        self._input_widget = XasMapInputWidget()
        self._input_widget.data_changed.connect(self._update_plots)
        self._last_read_future = None
        self._executor = ThreadPoolExecutor(max_workers=1)

        # XAS plotting
        plot_widget = qt.QWidget()
        plot_layout = qt.QVBoxLayout(plot_widget)
        plot_layout.setContentsMargins(0, 0, 0, 0)

        # Selected XAS map for plotting
        self._interpolator: Optional[XasMapInterpolator] = None

        # Plot XAS map energy dimension
        self._plot1d_widget = Plot1D()
        self._roi_widget = self._plot1d_widget.getCurvesRoiWidget()
        self._roi_widget.sigROISignal.connect(self._update_roi)
        self._roi_widget.sigROIWidgetSignal.connect(self._update_roi)

        # Plot XAS map spatial dimensions
        self._plot2d_widget = Plot2D()
        self._plot2d_widget.sigPlotSignal.connect(self._on_plot2d_click)
        plot_layout.addWidget(self._plot2d_widget)

        # XAS map spatial dimensions signal
        self._function_dropdown = qt.QComboBox()
        self._function_dropdown.currentIndexChanged.connect(
            self._on_function_dropdown_index_changed
        )
        self._reset_map_functions()
        plot_layout.addWidget(self._function_dropdown)

        # Bliss command of a point in the XAS map spatial dimensions
        self._bliss_command = qt.QLineEdit()
        self._bliss_command.setReadOnly(True)
        copy_bliss_command_button = qt.QToolButton()
        copy_bliss_command_button.setIcon(icons.getQIcon("edit-copy"))
        copy_bliss_command_button.setToolTip("Copy")
        copy_bliss_command_button.clicked.connect(self._copy_bliss_command_to_clipboard)
        click_layout = qt.QHBoxLayout()
        click_layout.addWidget(self._bliss_command)
        click_layout.addWidget(copy_bliss_command_button)
        plot_layout.addLayout(click_layout)

        # Arrange XAS map selection (left), 2D plot (right) and 1D plot (bottom)
        main_layout = qt.QVBoxLayout()

        splitter1 = qt.QSplitter()
        splitter1.setOrientation(qt.Qt.Horizontal)
        splitter1.addWidget(self._input_widget)
        splitter1.addWidget(plot_widget)

        splitter2 = qt.QSplitter()
        splitter2.setOrientation(qt.Qt.Vertical)
        splitter2.addWidget(splitter1)
        splitter2.addWidget(self._plot1d_widget)

        main_layout.addWidget(splitter2)

        central_widget = qt.QWidget()
        central_widget.setLayout(main_layout)
        self.setCentralWidget(central_widget)

    def _on_function_dropdown_index_changed(self, _):
        self._update_plots()

    def _update_plots(self, xasmap: dict = None):
        """This method is called when the XAS map is updated or the function is changed."""
        if xasmap is None:
            xasmap = self._input_widget.get_current_xasmap()
            if xasmap is None:
                return
        self._start_update_plots(xasmap)

    def _start_update_plots(self, xasmap: dict):
        if self._last_read_future is None:
            qt.QApplication.setOverrideCursor(qt.Qt.WaitCursor)

        log_arg = str(xasmap["filenames"])
        _logger.info("Load XAS map: %s", log_arg)

        future = self._executor.submit(
            read_xasmap,
            xasmap["filenames"],
            dim0_counter=xasmap["dim0_counter"],
            dim1_counter=xasmap["dim1_counter"],
            energy_counter=xasmap["energy_counter"],
            mu_counter=xasmap["mu_counter"],
            scan_ranges=xasmap["scan_ranges"],
            exclude_scans=xasmap["exclude_scans"],
        )
        previous_future = self._last_read_future
        self._last_read_future = future
        if previous_future is not None:
            previous_future.cancel()  # after updating _last_read_future

        def on_done(fut: Future) -> None:
            submitToQtMainThread(self._read_xasmap_done, fut, log_arg)

        future.add_done_callback(on_done)

    def _read_xasmap_done(self, future: Future, log_arg: str):
        try:
            result = future.result()
        except CancelledError:
            _logger.warning("XAS map loading cancelled: %s", log_arg)
            return
        except Exception:
            _logger.exception("XAS map loading failed: %s", log_arg)
            result = None
        else:
            _logger.info("XAS map loaded: %s", log_arg)
        self._plot_xasmap(result)

        if future is self._last_read_future:
            qt.QApplication.restoreOverrideCursor()
            self._last_read_future = None

    def _plot_xasmap(self, data: Optional[XasMapData]):
        self._interpolator = None
        try:
            if data is not None and not data.is_empty():
                self._interpolator = XasMapInterpolator(data)
        except Exception:
            _logger.exception("XAS data cannot be interpolated")
        finally:
            try:
                self._update_plot2d()
                self._update_plot1d()
            finally:
                self.plot_updated.emit()

    def _update_plot2d(self) -> None:
        if self._interpolator is None:
            self._plot2d_widget.clearImages()
            return

        selected_function = self._function_dropdown.currentText()
        img = self._interpolator.evaluate_as_map(self._map_functions[selected_function])

        origin = self._interpolator.x0_axis.min(), self._interpolator.x1_axis.min()
        scale = (
            self._interpolator.x0_axis[1] - self._interpolator.x0_axis[0],
            self._interpolator.x1_axis[1] - self._interpolator.x1_axis[0],
        )
        # Axis values are the center of the pixel.
        # The origin is the left corner of the first pixel.
        origin = origin[0] - scale[0] / 2, origin[1] - scale[1] / 2

        self._plot2d_widget.clearImages()
        title = selected_function
        self._plot2d_widget.setGraphTitle(title)

        self._plot2d_widget.addImage(
            img.T,  # X-axis is dimension 0
            legend=title,
            xlabel=self._interpolator.x0_axis_label,
            ylabel=self._interpolator.x1_axis_label,
            origin=origin,
            scale=scale,
        )

    def _update_plot1d(self) -> None:
        if self._interpolator is None:
            self._plot1d_widget.clearCurves()
            return
        signal = numpy.mean(self._interpolator.mu, axis=0)
        self._plot1d(signal, "Average of μ")

    def _plot1d(self, signal: numpy.ndarray, title: str) -> str:
        if self._interpolator is None:
            return
        self._plot1d_widget.addCurve(self._interpolator.energy_axis, signal, legend="μ")
        self._plot1d_widget.setGraphTitle(title)
        self._plot1d_widget.setGraphXLabel(self._interpolator.energy_label)
        self._plot1d_widget.setGraphYLabel("μ")

    def _reset_map_functions(self) -> None:
        self._map_functions: Dict[
            str, Callable[[numpy.ndarray, numpy.ndarray], float]
        ] = {"Sum of μ": _sum_mu, "Average of μ": _mean_mu}
        for name, roi in self._roi_widget.getRois().items():
            name = f"Sum of μ({name} - bkg)"
            self._map_functions[name] = _generate_sum_roi_mu(roi)
        self._reset_function_dropdown()

    def _reset_function_dropdown(self) -> None:
        with blockSignals(self._function_dropdown):
            current_text = self._function_dropdown.currentText()

            self._function_dropdown.clear()
            for name in self._map_functions:
                self._function_dropdown.addItem(name)

            index = self._function_dropdown.findText(current_text)
            index = max(index, 0)
            self._function_dropdown.setCurrentIndex(index)
            self._on_function_dropdown_index_changed(index)

    def _update_roi(self, _):
        self._reset_map_functions()
        self._update_plot2d()

    def _on_plot2d_click(self, event: dict) -> None:
        if (
            event["event"] == "mouseClicked"
            and event["button"] == "left"
            and self._interpolator is not None
        ):
            # X-axis is dimension 0
            pos_x0 = event["x"]
            pos_x1 = event["y"]
            mot_x0 = self._interpolator.x0_axis_name
            mot_x1 = self._interpolator.x1_axis_name

            command = f"umv({mot_x0}, {pos_x0:.6f}, {mot_x1}, {pos_x1:.6f})"
            self._bliss_command.setText(command)

            clipboard = qt.QApplication.clipboard()
            clipboard.setText(command)

            x, y, signal = self._interpolator.get_single_point_mu(pos_x0, pos_x1)
            self._plot1d(signal, f"μ at {mot_x0} = {x:.6f}, {mot_x1} = {y:.6f}")

    def _copy_bliss_command_to_clipboard(self):
        text = self._bliss_command.text()
        if text:
            clipboard = qt.QApplication.clipboard()
            clipboard.setText(text)

    def closeEvent(self, event):  # noqa N802
        self.save_settings()

    def save_settings(self) -> None:
        if self._settings is None:
            return
        self._save_settings_colormap()
        self._input_widget.save_settings()

    def load_settings(self) -> None:
        if self._settings is None:
            self._settings = get_settings()
        self._load_settings_colormap()
        self._input_widget.load_settings()

    def _save_settings_colormap(self) -> None:
        colormap = self._plot2d_widget.getDefaultColormap()
        self._settings.beginGroup("colormap")
        self._settings.setValue("default", colormap.saveState())
        self._settings.endGroup()

    def _load_settings_colormap(self) -> None:
        colormap = None
        self._settings.beginGroup("colormap")
        byte_array = self._settings.value("default", None)
        if byte_array is not None:
            try:
                colormap = Colormap()
                colormap.restoreState(byte_array)
            except Exception:
                _logger.debug("Failed loading color map", exc_info=True)
        self._settings.endGroup()

        if colormap:
            self._plot2d_widget.setDefaultColormap(colormap)

    def add_xasmap(self, *args, **kwargs) -> None:
        self._input_widget.add_xasmap(*args, **kwargs)


def _generate_sum_roi_mu(
    roi: ROI,
) -> Callable[[numpy.ndarray, numpy.ndarray], float]:
    roi_from = roi.getFrom()
    roi_to = roi.getTo()
    energy0 = min(roi_from, roi_to)
    energy1 = max(roi_from, roi_to)

    def _sum_roi(energy, mu):
        mask = (energy >= energy0) & (energy <= energy1)
        if not mask.any():
            return numpy.nan

        energy_roi = energy[mask]
        mu_roi = mu[mask]

        # Estimate background using linear interpolation between the endpoints
        idx0 = numpy.argmin(numpy.abs(energy - energy0))
        idx1 = numpy.argmin(numpy.abs(energy - energy1))
        e0, e1 = energy[idx0], energy[idx1]
        mu0, mu1 = mu[idx0], mu[idx1]

        # Linear background estimated across ROI
        background = mu0 + (mu1 - mu0) * (energy_roi - e0) / (e1 - e0 + 1e-12)
        corrected_mu = mu_roi - background
        corrected_mu[corrected_mu < 0] = 0

        return numpy.sum(corrected_mu)

    return _sum_roi


def _sum_mu(energy: numpy.ndarray, mu: numpy.ndarray) -> float:
    return numpy.sum(mu)


def _mean_mu(energy: numpy.ndarray, mu: numpy.ndarray) -> float:
    return numpy.mean(mu)



import os
from typing import List

import matplotlib
import matplotlib.pyplot as plt
import numpy
from ewokscore import Task

from ..plots.temperature import plot_temperature
from ..plots.xas import plot_xas


class ScanTemperaturePlot(
    Task,
    input_names=["temp_up_data", "temp_down_data"],
    optional_input_names=[
        "extend_plotrange_left",
        "extend_plotrange_right",
        "two_color_difference",
        "show",
        "pause",
        "output_directory",
        "figsize",
        "dpi",
        "fontsize",
    ],
    output_names=["filenames"],
):
    """Save laser-heated DAC temperature plots as images."""

    def run(self):
        self.prepare()
        self.outputs.filenames = self._plots()

    def _plots(self) -> List[str]:
        return self._temperature_plots(0)

    def _temperature_plots(self, plot_index: int) -> List[str]:
        out_filenames = []

        # Temperature plots
        extend_plotrange_left = self.get_input_value(
            "extend_plotrange_left", None
        )  # nm
        extend_plotrange_right = self.get_input_value(
            "extend_plotrange_right", None
        )  # nm
        two_color_difference = self.get_input_value("two_color_difference", None)  # nm

        filename = self.inputs.temp_up_data.filename
        scan_number = self.inputs.temp_up_data.scan_number
        basename = os.path.basename(self.inputs.temp_up_data.filename)
        dataset = os.path.splitext(basename)[0]
        for index in range(0, len(self.inputs.temp_up_data.epoch)):
            title = f"{dataset} #{scan_number} [{index}]"
            plot_temperature(
                self.inputs.temp_up_data,
                self.inputs.temp_down_data,
                index,
                title,
                extend_plotrange_left=extend_plotrange_left,
                extend_plotrange_right=extend_plotrange_right,
                two_color_difference=two_color_difference,
            )
            try:
                if not self.missing_inputs.output_directory:
                    out_filename = self._get_out_filename(
                        filename, scan_number, index + plot_index
                    )
                    if out_filename:
                        out_filenames.append(out_filename)
                        self.save_plot(out_filename)
                if self.inputs.show:
                    self.show_plot()
            finally:
                plt.close()

        return out_filenames

    def _get_out_filename(self, filename: int, scan_number: int, plotnr: int) -> str:
        output_directory = self.inputs.output_directory
        dataset = os.path.splitext(os.path.basename(filename))[0]
        output_directory = os.path.join(output_directory, dataset, f"scan{scan_number}")
        return os.path.join(output_directory, f"plot{plotnr}.png")

    def prepare(self) -> None:
        if not self.missing_inputs.fontsize:
            matplotlib.rc("font", size=self.inputs.fontsize)

    def save_plot(self, filename: str) -> None:
        path = os.path.dirname(filename)
        if path:
            os.makedirs(path, exist_ok=True)
        plt.gcf().savefig(filename, dpi=self.get_input_value("dpi", 150))

    def show_plot(self) -> None:
        if self.inputs.pause and numpy.isfinite(self.inputs.pause):
            plt.pause(self.inputs.pause)
        else:
            plt.show()


class XasTemperaturePlot(
    ScanTemperaturePlot,
    input_names=["xas_data"],
):
    """Save laser-heated DAC temperature XAS plots as images."""

    def _plots(self) -> List[str]:
        out_filenames = []
        out_filenames += self._xas_plots(len(out_filenames))
        out_filenames += self._temperature_plots(len(out_filenames))
        return out_filenames

    def _xas_plots(self, plot_index: int) -> List[str]:
        filename = self.inputs.xas_data.filename
        scan_number = self.inputs.xas_data.scan_number
        basename = os.path.basename(filename)
        dataset = os.path.splitext(basename)[0]
        title = f"{dataset} #{scan_number}"
        plot_xas(
            self.inputs.xas_data,
            self.inputs.temp_up_data,
            self.inputs.temp_down_data,
            title,
        )
        out_filename = self._get_out_filename(filename, scan_number, plot_index)
        self.save_plot(out_filename)
        return [out_filename]



import os

import numpy
import pytest
from ewokscore import execute_graph


@pytest.mark.parametrize("with_xas", [True, False])
def test_scan_saving(xanes_filename, tmp_path, with_xas):
    if with_xas:
        read_task = "ewoksid24.tasks.read.XasTemperatureRead"
        plot_task = "ewoksid24.tasks.plot.XasTemperaturePlot"
        nplots = 5
    else:
        read_task = "ewoksid24.tasks.read.ScanTemperatureRead"
        plot_task = "ewoksid24.tasks.plot.ScanTemperaturePlot"
        nplots = 4

    workflow = {
        "graph": {"id": "test_temperature_task"},
        "nodes": [
            {
                "id": 0,
                "task_identifier": read_task,
                "task_type": "class",
                "default_inputs": [
                    {"name": "filename", "value": xanes_filename},
                    {"name": "scan_number", "value": 39},
                    {"name": "retry_timeout", "value": 0},
                ],
            },
            {
                "id": 1,
                "task_identifier": plot_task,
                "task_type": "class",
                "default_inputs": [
                    {"name": "output_directory", "value": str(tmp_path)}
                ],
            },
        ],
        "links": [{"source": 0, "target": 1, "map_all_data": True}],
    }

    result = execute_graph(workflow)

    filenames = result["filenames"]
    assert len(filenames) == nplots
    for filename in filenames:
        assert filename.endswith(".png")
        assert os.path.isfile(filename)


@pytest.mark.parametrize("with_xas", [True, False])
def test_fitting(xanes_filename, with_xas):
    if with_xas:
        read_task = "ewoksid24.tasks.read.XasTemperatureRead"
    else:
        read_task = "ewoksid24.tasks.read.ScanTemperatureRead"

    workflow = {
        "graph": {"id": "test_temperature_task"},
        "nodes": [
            {
                "id": 0,
                "task_identifier": read_task,
                "task_type": "class",
                "default_inputs": [
                    {"name": "filename", "value": xanes_filename},
                    {"name": "scan_number", "value": 39},
                    {"name": "retry_timeout", "value": 0},
                ],
            },
            {
                "id": 1,
                "task_identifier": "ewoksid24.tasks.fit.PlanckRadianceFit",
                "task_type": "class",
            },
        ],
        "links": [
            {
                "source": 0,
                "target": 1,
                "data_mapping": [
                    {"source_output": "temp_up_data", "target_input": "temp_data"}
                ],
            }
        ],
    }

    result = execute_graph(
        workflow,
        outputs=[{"id": 0, "name": "temp_up_data"}, {"id": 1, "name": "temp_data"}],
    )

    temperature0 = result["temp_up_data"].planck_temperature
    temperature1 = result["temp_data"].planck_temperature

    # Test: changed but close
    numpy.testing.assert_raises(
        AssertionError, numpy.testing.assert_array_equal, temperature0, temperature1
    )
    numpy.testing.assert_allclose(temperature0, temperature1, atol=1e-3)



from typing import Callable
from typing import Tuple

import numpy
from ewoksfluo.math.regular_grid import ScatterDataInterpolator
from scipy.interpolate import interp1d

from ..io.xasmap import XasMapData


class XasMapInterpolator:

    def __init__(self, data: XasMapData) -> None:
        self._energy_units = data.energy_units
        if data.energy.ndim == 2:
            self._energy_axis, self._mu = interpolate_energy(data)
        elif data.energy.ndim == 1:
            self._energy_axis = data.energy
            self._mu = data.mu
        else:
            raise ValueError("energy field must be 1D or 2D")
        self._interpolator = ScatterDataInterpolator(
            [data.x0, data.x1],
            [data.x0_name, data.x1_name],
            [data.x0_units, data.x1_units],
        )

    @property
    def energy_axis(self):
        return self._energy_axis

    @property
    def energy_label(self) -> str:
        name = "Energy"
        units = self._energy_units
        if units:
            return f"{name} ({units})"
        else:
            return name

    @property
    def x0_axis(self) -> numpy.ndarray:
        return self._interpolator.grid_axes[0]

    @property
    def x1_axis(self) -> numpy.ndarray:
        return self._interpolator.grid_axes[1]

    @property
    def x0_axis_name(self) -> str:
        return self._interpolator.axes_names[0]

    @property
    def x1_axis_name(self) -> str:
        return self._interpolator.axes_names[1]

    @property
    def x0_axis_label(self) -> str:
        return self._axis_label(0)

    @property
    def x1_axis_label(self) -> str:
        return self._axis_label(1)

    @property
    def mu(self):
        return self._mu

    def _axis_label(self, i: int) -> str:
        name = self._interpolator.axes_names[i]
        units = self._interpolator.units

        if not units:
            return name

        if not isinstance(units, str):
            units = units[i]

        if not units or units == "dimensionless":
            return name

        return f"{name} ({units})"

    def evaluate_as_map(
        self, xas_func: Callable[[numpy.ndarray, numpy.ndarray], float]
    ) -> numpy.ndarray:
        energy = self.energy_axis
        values = numpy.array([xas_func(energy, mu) for mu in self._mu])
        return self._interpolator.regrid(values)

    def get_single_point_mu(
        self, pos_x0: float, pos_x1: float
    ) -> Tuple[float, float, numpy.ndarray]:
        scatter0, scatter1 = self._interpolator.scatter_coordinates.T
        distance = numpy.abs(scatter0 - pos_x0) + numpy.abs(scatter1 - pos_x1)
        idx = numpy.argmin(distance)
        return scatter0[idx], scatter1[idx], self._mu[idx]


def interpolate_energy(data: XasMapData) -> Tuple[numpy.ndarray, numpy.ndarray]:
    """
    Interpolates all mu spectra to a common energy axis.

    :param data: Input XasMapData with possibly different energy axes per point.
    :returns: Shared energy axis and interpolated mu.
    """
    e0 = data.energy[:, 0].max()
    e1 = data.energy[:, -1].min()
    en = data.energy.shape[1]
    energy = numpy.linspace(e0, e1, en)

    mu_interp = numpy.empty((data.mu.shape[0], len(energy)))
    for i in range(data.mu.shape[0]):
        interp = interp1d(
            data.energy[i], data.mu[i], bounds_error=False, fill_value=numpy.nan
        )
        mu_interp[i] = interp(energy)

    return energy, mu_interp



from dataclasses import dataclass

import numpy

from .bliss import read_counters


@dataclass
class XasData:
    filename: str
    scan_number: int
    epoch: numpy.ndarray
    energy: numpy.ndarray  # keV
    mu: numpy.ndarray


def read_xas_data(
    filename: str, scan_number: int, energy_name: str, mu_name: str, **retry_options
) -> XasData:
    counters = ["epoch", energy_name, mu_name]
    epoch, energy, mu = read_counters(
        filename, scan_number, 1, counters, **retry_options
    )
    return XasData(
        filename=filename, scan_number=scan_number, epoch=epoch, energy=energy, mu=mu
    )



from silx.gui import qt

from ...gui.xasmap_viewer import XasMapViewer


def test_viewer_initializes(qtbot):
    widget = XasMapViewer()
    qtbot.addWidget(widget)
    widget.show()

    assert widget.windowTitle() == "XAS Map Viewer"
    assert isinstance(widget._plot2d_widget, qt.QWidget)
    assert widget._function_dropdown.count() > 0


def test_add_xasmap(qtbot, qapp, id24_dcm_xasmap):
    widget = XasMapViewer()
    qtbot.addWidget(widget)

    widget.show()

    with qtbot.waitSignal(widget.plot_updated, timeout=30000):
        widget.add_xasmap(
            filenames=id24_dcm_xasmap["filenames"],
            dim0_counter=id24_dcm_xasmap["dim0_counter"],
            dim1_counter=id24_dcm_xasmap["dim1_counter"],
            energy_counter=id24_dcm_xasmap["energy_counter"],
            mu_counter=id24_dcm_xasmap["mu_counter"],
        )

    images = widget._plot2d_widget.getAllImages()
    curves = widget._plot1d_widget.getAllCurves()

    assert len(images) == 1
    assert len(curves) == 1

    assert images[0].getName() == "Sum of μ"
    assert curves[0].getName() == "μ"

    qapp.processEvents()



from typing import List

import numpy
from blissdata.h5api import dynamic_hdf5


def read_counters(
    filename: str,
    scan_number: int,
    subscan_number: int,
    counters: List[str],
    **retry_options,
) -> List[numpy.ndarray]:
    with dynamic_hdf5.File(filename, mode="r", **retry_options) as nxroot:
        scan = nxroot[f"{scan_number}.{subscan_number}"]
        _ = scan["end_time"]  # wait until scan is finished
        measurement = scan["measurement"]
        data = [measurement[name][()] for name in counters]
    nmin = min(len(values) for values in data)
    return [values[:nmin] for values in data]



import json
import os
from typing import List
from typing import Optional
from typing import Tuple

from silx.gui import icons
from silx.gui import qt

from .settings import get_settings


class XasMapInputWidget(qt.QWidget):
    # Signal to notify the main window when the input is changed
    data_changed = qt.pyqtSignal(dict)

    def __init__(self, parent=None):
        super().__init__(parent)
        self._settings: Optional[qt.QSettings] = None

        # Buttons to add/remove XAS maps
        xasmap_box = qt.QGroupBox("XAS maps:")
        xasmap_layout = qt.QVBoxLayout(xasmap_box)

        # List of XAS maps
        self._xasmaps_widget = qt.QListWidget()
        self._xasmaps_widget.setSelectionMode(qt.QListWidget.SingleSelection)
        self._xasmaps_widget.itemSelectionChanged.connect(self._select_xasmap)
        self._xasmaps_widget.setContextMenuPolicy(qt.Qt.CustomContextMenu)
        self._xasmaps_widget.customContextMenuRequested.connect(
            self._xasmaps_context_menu
        )

        delete_button = qt.QPushButton("Remove")
        delete_button.clicked.connect(self._remove_selected_xasmap)

        # Layout to hold the current XAS maps
        xasmap_layout.addWidget(self._xasmaps_widget)
        xasmap_layout.addWidget(delete_button, alignment=qt.Qt.AlignRight)

        # Layout to hold the XAS map parameters
        parameters_box = qt.QGroupBox("Parameters:")
        parameters_layout = qt.QVBoxLayout(parameters_box)

        # XAS map parameters: files
        self._last_dir = os.path.expanduser("~")

        replace_files_button = qt.QToolButton()
        replace_files_button.setIcon(icons.getQIcon("document-open"))
        replace_files_button.setToolTip("Replace file")
        replace_files_button.clicked.connect(self._replace_files)

        add_files_button = qt.QToolButton()
        add_files_button.setIcon(icons.getQIcon("folder"))
        add_files_button.setToolTip("Add files")
        add_files_button.clicked.connect(self._add_files)

        self._filenames_widget = qt.QListWidget()
        self._filenames_widget.setSelectionMode(qt.QListWidget.ExtendedSelection)
        self._filenames_widget.setContextMenuPolicy(qt.Qt.CustomContextMenu)
        self._filenames_widget.customContextMenuRequested.connect(
            self._filenames_context_menu
        )
        self._filenames_widget.itemChanged.connect(self._parameters_changed)
        self._filenames_widget.model().rowsInserted.connect(self._parameters_changed)
        self._filenames_widget.model().rowsRemoved.connect(self._parameters_changed)

        file_layout = qt.QHBoxLayout()
        file_layout.addWidget(self._filenames_widget)
        file_selection_layout = qt.QVBoxLayout()
        file_selection_layout.addWidget(replace_files_button)
        file_selection_layout.addWidget(add_files_button)
        file_layout.addLayout(file_selection_layout)
        file_row_widget = qt.QWidget()
        file_row_widget.setLayout(file_layout)

        # XAS map parameters: all others
        parameter_form = qt.QFormLayout()
        parameters_layout.addLayout(parameter_form)

        self._dim0_counter_input = qt.QLineEdit()
        self._dim1_counter_input = qt.QLineEdit()
        self._energy_counter_input = qt.QLineEdit()
        self._mu_counter_input = qt.QLineEdit()
        self._scan_range_input = qt.QLineEdit()
        self._exclude_scans_input = qt.QLineEdit()

        self._dim0_counter_input.textChanged.connect(self._parameters_changed)
        self._dim1_counter_input.textChanged.connect(self._parameters_changed)
        self._energy_counter_input.textChanged.connect(self._parameters_changed)
        self._mu_counter_input.textChanged.connect(self._parameters_changed)
        self._scan_range_input.textChanged.connect(self._parameters_changed)
        self._exclude_scans_input.textChanged.connect(self._parameters_changed)

        parameter_form.addRow("Files", file_row_widget)
        parameter_form.addRow("X-Axis", self._dim0_counter_input)
        parameter_form.addRow("Y-Axis", self._dim1_counter_input)
        parameter_form.addRow("Energy", self._energy_counter_input)
        parameter_form.addRow("Mu", self._mu_counter_input)
        parameter_form.addRow("Scan ranges", self._scan_range_input)
        parameter_form.addRow("Exclude scans", self._exclude_scans_input)

        button_layout = qt.QHBoxLayout()
        parameters_layout.addLayout(button_layout)

        create_button = qt.QPushButton("New")
        create_button.clicked.connect(self._create_xasmap)

        self._modify_button = qt.QPushButton("Update")
        self._modify_button.setEnabled(False)
        self._modify_button.clicked.connect(self._modify_xasmap)

        self._reset_button = qt.QPushButton("Reset")
        self._reset_button.setEnabled(False)
        self._reset_button.clicked.connect(self._select_xasmap)

        button_layout.addWidget(create_button)
        button_layout.addWidget(self._modify_button)
        button_layout.addWidget(self._reset_button)

        # Add all components to the main layout
        main_layout = qt.QVBoxLayout(self)
        main_layout.setContentsMargins(0, 0, 0, 0)
        main_layout.addWidget(xasmap_box)
        main_layout.addWidget(parameters_box)

    def add_xasmap(
        self,
        filenames: Optional[List[str]] = None,
        dim0_counter: Optional[str] = None,
        dim1_counter: Optional[str] = None,
        energy_counter: Optional[str] = None,
        mu_counter: Optional[str] = None,
        scan_ranges: Optional[List[List[int]]] = None,
        exclude_scans: Optional[List[int]] = None,
    ) -> None:
        do_create = False
        if filenames:
            self._set_filenames(filenames)
            do_create = True

        if dim0_counter is not None:
            self._set_parameter_as_string(self._dim0_counter_input, dim0_counter)
            do_create = True

        if dim1_counter is not None:
            self._set_parameter_as_string(self._dim1_counter_input, dim1_counter)
            do_create = True

        if energy_counter is not None:
            self._set_parameter_as_string(self._energy_counter_input, energy_counter)
            do_create = True

        if mu_counter is not None:
            self._set_parameter_as_string(self._mu_counter_input, mu_counter)
            do_create = True

        if scan_ranges:
            scan_ranges = _parse_list_of_lists_of_ints(scan_ranges, two_tuple=True)
            self._set_scan_ranges(scan_ranges)
            do_create = True
        elif filenames:
            self._set_scan_ranges(None)

        if exclude_scans:
            exclude_scans = _parse_list_of_lists_of_ints(exclude_scans, two_tuple=False)
            self._set_exclude_scans(exclude_scans)
            do_create = True
        elif filenames:
            self._set_exclude_scans(None)

        if do_create:
            self._create_xasmap()

    def get_current_xasmap(self) -> Optional[dict]:
        return self._get_selected_xasmap()

    def save_settings(self) -> None:
        if self._settings is None:
            return
        xasmaps = self._get_xasmaps()
        xasmap_index = self._xasmaps_widget.currentRow()

        self._settings.beginGroup("XasMapInputWidget")
        self._settings.setValue("xasmaps", xasmaps)
        self._settings.setValue("xasmap_index", xasmap_index)
        self._settings.endGroup()

    def load_settings(self) -> None:
        """Load input sets from QSettings and restore the form fields."""
        if self._settings is None:
            self._settings = get_settings()

        self._settings.beginGroup("XasMapInputWidget")
        xasmaps = self._settings.value("xasmaps", None)
        xasmap_index = int(self._settings.value("xasmap_index", -1))
        self._settings.endGroup()

        self._set_xasmaps(xasmaps)
        if xasmap_index != -1:
            self._xasmaps_widget.setCurrentRow(xasmap_index)

    def _add_files(self) -> None:
        """Open a QFileDialog to select multiple files to add to the current ones."""
        filenames, _ = qt.QFileDialog.getOpenFileNames(
            self,
            "Select XAS Data Files",
            self._last_dir,
            "HDF5 Files (*.h5);;All Files (*)",
        )
        self._add_filenames(filenames)

    def _replace_files(self) -> None:
        """Open a QFileDialog to select a single file to replace the current ones."""
        filename, _ = qt.QFileDialog.getOpenFileName(
            self,
            "Select XAS Data File",
            self._last_dir,
            "HDF5 Files (*.h5);;All Files (*)",
        )
        self._set_filenames([filename])

    def _get_filenames(self) -> List[str]:
        nitems = self._filenames_widget.count()
        return [
            self._filenames_widget.item(i).data(qt.Qt.UserRole) for i in range(nitems)
        ]

    def _set_filenames(self, filenames: Optional[List[Tuple[str, str]]]) -> None:
        self._filenames_widget.clear()
        if not filenames:
            return
        for filename in filenames:
            _ = self._add_filename(filename)

    def _add_filenames(self, filenames: Optional[List[str]]) -> None:
        if not filenames:
            return
        for filename in filenames:
            self._add_filename(filename)
        self.save_settings()

    def _add_filename(self, filename: str) -> None:
        self._last_dir = os.path.dirname(filename)
        filelabel = os.path.basename(filename)
        filename = os.path.abspath(filename)

        item = qt.QListWidgetItem(filelabel)
        item.setData(qt.Qt.UserRole, filename)
        self._filenames_widget.addItem(item)

    def _filenames_context_menu(self, pos) -> None:
        """Show the context menu for right-click on the file list."""
        context_menu = qt.QMenu(self)

        duplicate_action = context_menu.addAction("Duplicate")
        remove_action = context_menu.addAction("Remove")
        action = context_menu.exec_(self._filenames_widget.mapToGlobal(pos))

        if action == remove_action:
            self._remove_selected_filenames()
        elif action == duplicate_action:
            self._duplicate_selected_filenames()

    def _remove_selected_filenames(self) -> None:
        """Remove the selected files from the file list."""
        selected_items = self._filenames_widget.selectedItems()
        if selected_items is None:
            return
        for selected_item in selected_items:
            index = self._filenames_widget.row(selected_item)
            self._filenames_widget.takeItem(index)
        self.save_settings()

    def _duplicate_selected_filenames(self) -> None:
        """Duplicate the selected files in the file list."""
        selected_items = self._filenames_widget.selectedItems()
        if selected_items is None:
            return
        for selected_item in selected_items:
            filename = selected_item.data(qt.Qt.UserRole)
            self._add_filename(filename)
        self.save_settings()

    def _get_xasmaps(self) -> List[dict]:
        nitems = self._xasmaps_widget.count()
        return [
            self._xasmaps_widget.item(i).data(qt.Qt.UserRole) for i in range(nitems)
        ]

    def _set_xasmaps(self, xasmaps: Optional[List[dict]]) -> None:
        self._xasmaps_widget.clear()
        if xasmaps:
            for xasmap in xasmaps:
                self._add_xasmap(xasmap)

    def _add_xasmap(self, xasmap: dict) -> None:
        label = self._get_xasmap_id(xasmap)
        item = qt.QListWidgetItem(label)
        item.setData(qt.Qt.UserRole, xasmap)
        self._xasmaps_widget.addItem(item)

    def _select_last_xasmap(self) -> None:
        count = self._xasmaps_widget.count()
        if count > 0:
            self._xasmaps_widget.setCurrentRow(count - 1)

    def _get_xasmap_id(self, xasmap: dict) -> str:
        filenames = xasmap.get("filenames", [])
        scan_ranges = xasmap.get("scan_ranges", [])
        if not scan_ranges:
            scan_ranges = [None] * len(filenames)

        names = []
        for filename, scan_range in zip(filenames, scan_ranges):
            name = os.path.basename(filename)

            if scan_range:
                name = f"{name}: {scan_range[0]} -> {scan_range[1]}"
            names.append(name)

        return ", ".join(names)

    def _get_selected_xasmap(self) -> Optional[dict]:
        selected_item = self._xasmaps_widget.currentItem()
        if selected_item:
            return selected_item.data(qt.Qt.UserRole)

    def _xasmaps_context_menu(self, pos) -> None:
        """Show the context menu for right-click on the XAS map list."""
        context_menu = qt.QMenu(self)

        remove_action = context_menu.addAction("Remove")
        action = context_menu.exec_(self._xasmaps_widget.mapToGlobal(pos))

        if action == remove_action:
            self._remove_selected_xasmap()

    def _remove_selected_xasmap(self) -> None:
        """Remove the current XAS map."""
        selected_index = self._xasmaps_widget.currentRow()
        if selected_index != -1:
            self._xasmaps_widget.takeItem(selected_index)
            self.save_settings()
            self._select_xasmap()

    def _create_xasmap(self) -> None:
        """Add the current parameters as a new XAS map."""
        xasmap = self._get_edited_xasmap()
        if xasmap:
            self._add_xasmap(xasmap)
            self.save_settings()
            self._select_last_xasmap()

    def _emit_xasmap(self, xasmap: dict) -> None:
        self.data_changed.emit(xasmap)

    def _modify_xasmap(self) -> None:
        selected_item = self._xasmaps_widget.currentItem()
        if selected_item is None:
            return
        xasmap = self._get_edited_xasmap()
        if xasmap is None:
            return
        selected_item.setData(qt.Qt.UserRole, xasmap)
        label = self._get_xasmap_id(xasmap)
        selected_item.setText(label)
        self.save_settings()
        self._select_xasmap()

    def _get_edited_xasmap(self) -> Optional[dict]:
        try:
            filenames = self._get_xasmap_filenames()
            scan_ranges = self._get_scan_ranges()
            exclude_scans = self._get_exclude_scans()
            dim0_counter = self._get_parameter_as_string(
                self._dim0_counter_input, "X-Axis"
            )
            dim1_counter = self._get_parameter_as_string(
                self._dim1_counter_input, "Y-Axis"
            )
            energy_counter = self._get_parameter_as_string(
                self._energy_counter_input, "Energy"
            )
            mu_counter = self._get_parameter_as_string(self._mu_counter_input, "Mu")
        except _Return:
            return None

        return {
            "filenames": filenames,
            "dim0_counter": dim0_counter,
            "dim1_counter": dim1_counter,
            "energy_counter": energy_counter,
            "mu_counter": mu_counter,
            "scan_ranges": scan_ranges,
            "exclude_scans": exclude_scans,
        }

    def _select_xasmap(self) -> None:
        xasmap = self._get_selected_xasmap()
        if not xasmap:
            return
        self._set_parameter_as_string(
            self._dim0_counter_input, xasmap.get("dim0_counter")
        )
        self._set_parameter_as_string(
            self._dim1_counter_input, xasmap.get("dim1_counter")
        )
        self._set_parameter_as_string(
            self._energy_counter_input, xasmap.get("energy_counter")
        )
        self._set_parameter_as_string(self._mu_counter_input, xasmap.get("mu_counter"))
        self._set_scan_ranges(xasmap.get("scan_ranges"))
        self._set_exclude_scans(xasmap.get("exclude_scans"))
        self._set_filenames(xasmap.get("filenames"))
        self._update_button_states(False)
        self._emit_xasmap(xasmap)

    def _get_xasmap_filenames(self) -> List[str]:
        filenames = self._get_filenames()
        if not filenames:
            qt.QMessageBox.warning(
                self, "No Files Selected", "Please select at least one file."
            )
            raise _Return
        return filenames

    def _get_parameter_as_string(self, w: qt.QLineEdit, name: str) -> str:
        value = w.text()
        if not value:
            qt.QMessageBox.warning(self, "Invalid Input", f"'{name}' is not defined")
            raise _Return
        return value

    def _set_parameter_as_string(self, w: qt.QLineEdit, value: Optional[str]) -> None:
        if value is None:
            w.setText("")
        else:
            w.setText(str(value))

    def _get_scan_ranges(self) -> Optional[List[List[int]]]:
        scan_ranges = self._scan_range_input.text() or None
        if not scan_ranges:
            return None
        try:
            return _parse_list_of_lists_of_ints(scan_ranges, two_tuple=True)
        except (TypeError, json.JSONDecodeError) as e:
            qt.QMessageBox.warning(
                self,
                "Invalid Value",
                f"The scan ranges must be a list of lists of two integers ({e}).",
            )
            raise _Return

    def _set_scan_ranges(self, scan_ranges: Optional[List[List[int]]]) -> None:
        if scan_ranges:
            self._scan_range_input.setText(json.dumps(scan_ranges))
        else:
            self._scan_range_input.setText("")

    def _get_exclude_scans(self) -> Optional[List[List[int]]]:
        exclude_scans = self._exclude_scans_input.text() or None
        if not exclude_scans:
            return None
        try:
            return _parse_list_of_lists_of_ints(exclude_scans, two_tuple=False)
        except (TypeError, json.JSONDecodeError) as e:
            qt.QMessageBox.warning(
                self,
                "Invalid Value",
                f"The scans to exclude must be a list of lists of integers ({e}).",
            )
            raise _Return

    def _set_exclude_scans(self, exclude_scans: Optional[List[List[int]]]) -> None:
        if exclude_scans:
            self._exclude_scans_input.setText(json.dumps(exclude_scans))
        else:
            self._exclude_scans_input.setText("")

    def _parameters_changed(self):
        self._update_button_states(True)

    def _update_button_states(self, enable: bool) -> None:
        self._modify_button.setEnabled(enable)
        self._reset_button.setEnabled(enable)


def _parse_list_of_lists_of_ints(
    value: str, two_tuple: bool
) -> Optional[List[List[int]]]:
    value = json.loads(value)
    if value is None:
        return None
    if isinstance(value, int):
        if two_tuple:
            value = [[value, value]]
        else:
            value = [[value]]
    if not isinstance(value, list):
        raise TypeError("Not a list")
    if len({isinstance(v, int) for v in value}) == 2:
        raise TypeError("List is a mixture of integers and other types")
    if all(isinstance(v, int) for v in value):
        value = [value]
    for lst in value:
        if not all(isinstance(v, int) for v in lst):
            raise TypeError("Sub-list must contain integers")
        if two_tuple and len(lst) != 2:
            raise TypeError("Sub-list only has two values (first and last scan number)")
    return value


class _Return(Exception):
    pass



import os

import h5py
import numpy
import pytest

from .data import RESOURCE_ROOT


@pytest.fixture
def xanes_filename():
    return os.path.join(RESOURCE_ROOT, "xanes.h5")


@pytest.fixture
def id24_dcm_xasmap(tmp_path):
    x = numpy.linspace(-1, 2, 10)
    y = numpy.linspace(3, 2, 9)

    xv, yv = numpy.meshgrid(x, y)
    xv = xv.flatten()
    yv = yv.flatten()

    energy = numpy.linspace(7, 7.1, 20)
    mu = numpy.linspace(0, 1, 20)

    filename = str(tmp_path / "id24_dcm.h5")
    with h5py.File(filename, "w") as nxroot:
        for i in range(len(xv)):
            nxroot[f"/{i}.1/instrument/positioners/x"] = xv[i]
            nxroot[f"/{i}.1/instrument/positioners/x"].attrs["units"] = "um"
            nxroot[f"/{i}.1/instrument/positioners/y"] = yv[i]
            nxroot[f"/{i}.1/instrument/positioners/y"].attrs["units"] = "um"
            nxroot[f"/{i}.1/measurement/energy"] = energy
            nxroot[f"/{i}.1/measurement/energy"].attrs["units"] = "keV"
            nxroot[f"/{i}.1/measurement/mu"] = mu + 0.01 * i

    return {
        "filenames": [filename],
        "dim0_counter": "x",
        "dim1_counter": "y",
        "mu_counter": "mu",
        "energy_counter": "energy",
    }


@pytest.fixture
def id24_ed_xasmap(tmp_path):
    x = numpy.linspace(-1, 2, 10)
    y = numpy.linspace(3, 2, 9)

    xv, yv = numpy.meshgrid(x, y)
    xv = xv.flatten()
    yv = yv.flatten()

    energy = numpy.linspace(7, 7.1, 20)[None, :]
    mu = numpy.linspace(0, 1, 20)
    mu = numpy.arange(len(xv))[:, None] * mu[None, :]

    filename = str(tmp_path / "id24_ed.h5")
    with h5py.File(filename, "w") as nxroot:
        nxroot["/1.1/instrument/positioners/x"] = xv
        nxroot["/1.1/instrument/positioners/x"].attrs["units"] = "um"
        nxroot["/1.1/instrument/positioners/y"] = yv
        nxroot["/1.1/instrument/positioners/y"].attrs["units"] = "um"
        nxroot["/1.1/measurement/energy"] = energy
        nxroot["/1.1/measurement/energy"].attrs["units"] = "keV"
        nxroot["/1.1/measurement/mu"] = mu

    return {
        "filenames": [filename],
        "dim0_counter": "x",
        "dim1_counter": "y",
        "mu_counter": "mu",
        "energy_counter": "energy",
    }



import matplotlib.pyplot as plt
import numpy

from ..io.temperature import TemperatureData
from ..io.xas import XasData


def plot_xas(
    xas_data: XasData,
    temp_up_data: TemperatureData,
    temp_down_data: TemperatureData,
    title: str,
    figsize=None,
) -> None:
    epoch_min = min(
        [min(xas_data.epoch), min(temp_up_data.epoch), min(temp_down_data.epoch)]
    )

    fig, ax1 = plt.subplots(figsize=figsize)
    fig.suptitle(title)
    xas_color = "k"
    ax1.plot(xas_data.epoch - epoch_min, xas_data.mu, color=xas_color)
    ax1.set_xlabel("time (sec)")
    ax1.set_ylabel("mu", color=xas_color)
    ax1.tick_params(axis="y", labelcolor=xas_color)

    ax2 = ax1.twinx()
    temp_color = "tab:red"
    ax2.set_ylabel("Temperature (K)", color=temp_color)

    temp_up_avg = numpy.average(temp_up_data.planck_temperature)
    ax2.plot(
        temp_up_data.epoch - epoch_min,
        temp_up_data.planck_temperature,
        "-o",
        color="green",
        label=f"<{temp_up_data.label}> = {temp_up_avg:.0f} K",
    )

    temp_ds_avg = numpy.average(temp_down_data.planck_temperature)
    ax2.plot(
        temp_down_data.epoch - epoch_min,
        temp_down_data.planck_temperature,
        "-o",
        color="blue",
        label=f"<{temp_down_data.label}> = {temp_ds_avg:.0f} K",
    )
    ax2.tick_params(axis="y", labelcolor=temp_color)
    ax2.legend()

    fig.tight_layout()



from typing import Optional

import matplotlib.pyplot as plt
import numpy

from ..io.temperature import TemperatureData


def plot_temperature(
    temp_up_data: TemperatureData,
    temp_down_data: TemperatureData,
    index: int,
    title: str,
    extend_plotrange_left: Optional[float] = None,
    extend_plotrange_right: Optional[float] = None,
    two_color_difference: Optional[float] = None,
    figsize=None,
):
    """
    :param temp_up_data:
    :param temp_down_data:
    :param index: point index in the scan
    :param title: plot title
    :param extend_plotrange_left: extend the plot range with respect to the fit range on the left in nm
    :param extend_plotrange_right: extend the plot range with respect to the fit range on the right in nm
    :param two_color_difference: compare wavelengths with this difference
    :param figsize: plot size
    """
    if extend_plotrange_left is None:
        extend_plotrange_left = -15  # nm
    if extend_plotrange_right is None:
        extend_plotrange_right = 50  # nm
    if two_color_difference is None:
        two_color_difference = 42  # nm
    fig, ax = plt.subplots(nrows=3, ncols=2, sharex=True, figsize=figsize)
    try:
        fig.suptitle(title)
        _plot_column(
            0,
            ax,
            temp_up_data,
            index,
            extend_plotrange_left,
            extend_plotrange_right,
            two_color_difference,
        )
        _plot_column(
            1,
            ax,
            temp_down_data,
            index,
            extend_plotrange_left,
            extend_plotrange_right,
            two_color_difference,
        )
        fig.tight_layout()
    except Exception:
        plt.close()
        raise


def _plot_column(
    column: int,
    ax: plt.Axes,
    temp_data: TemperatureData,
    index: int,
    extend_plotrange_left: float,
    extend_plotrange_right: float,
    two_color_difference: float,
):
    fit_slice = temp_data.planck_fit_slice

    wavelength = temp_data.wavelength[index]
    response = temp_data.response[index]
    planck_data = temp_data.planck_data[index]
    planck_fit = temp_data.planck_fit[index, fit_slice]
    title = f"{temp_data.label} = {temp_data.planck_temperature[index]:.0f} K"

    # Fit and plot range
    imin = fit_slice.start
    imax = fit_slice.stop - 1
    wavelength_fit_min = wavelength[imin]
    wavelength_fit_max = wavelength[imax]
    xmin = wavelength_fit_min + extend_plotrange_left
    xmax = wavelength_fit_max + extend_plotrange_right

    # Raw data
    ax[0, column].set_title(title)
    ax[0, column].plot(wavelength, response)
    ax[0, column].set_xlim(xmin, xmax)

    # Planck fit
    ax[1, column].plot(wavelength, planck_data)
    ax[1, column].plot(wavelength[fit_slice], planck_fit)
    ax[1, column].set_xlim(xmin, xmax)
    ax[1, column].set_ylim(0, 1.1 * max(planck_data[fit_slice]))

    # Two-color difference
    if not two_color_difference:
        two_color_difference = 0.1 * (wavelength_fit_max - wavelength_fit_min)
    dwavelength = numpy.median(numpy.diff(wavelength))
    d = max(int(two_color_difference / dwavelength), 1)
    diff_start = imin
    diff_stop = min(imax + d + 1, len(wavelength))
    d = diff_stop - imax - 1
    diff_slice = slice(diff_start, diff_stop)

    x = wavelength[diff_slice]
    y = numpy.log(planck_data[diff_slice] / x**5)
    num = y[:-d] - y[d:]
    denom = 1e9 / x[:-d] - 1e9 / x[d:]
    temp_col = num / denom

    ax[2, column].plot(x[:-d], temp_col)
    ax[2, column].set(xlabel="wavelength (nm)")
    ax[2, column].set_xlim(xmin, xmax)



from ewokscore import Task

from ..io.temperature import read_temperature_data
from ..io.xas import read_xas_data


class ScanTemperatureRead(
    Task,
    input_names=["filename", "scan_number"],
    optional_input_names=["subscan_number", "retry_timeout", "retry_period"],
    output_names=["temp_up_data", "temp_down_data"],
):
    """Read laser-heated DAC scan with temperature data."""

    def run(self):
        filename = self.inputs.filename
        scan_number = self.inputs.scan_number
        subscan_number = self.get_input_value("subscan_number", 2)
        retry_options = self._get_retry_options()

        self.outputs.temp_up_data = read_temperature_data(
            filename, scan_number, subscan_number, "up", "T_US", **retry_options
        )
        self.outputs.temp_down_data = read_temperature_data(
            filename, scan_number, subscan_number, "down", "T_DS", **retry_options
        )

    def _get_retry_options(self) -> dict:
        return {
            "retry_timeout": self.get_input_value("retry_timeout", 60),
            "retry_period": self.get_input_value("retry_period", 0.5),
        }


class XasTemperatureRead(
    ScanTemperatureRead,
    optional_input_names=["energy_name", "mu_name"],
    output_names=["xas_data"],
):
    """Read laser-heated DAC XAS scan with temperature data."""

    def run(self):
        filename = self.inputs.filename
        scan_number = self.inputs.scan_number
        retry_options = self._get_retry_options()

        energy_name = self.get_input_value("energy_name", "energy_enc")
        mu_name = self.get_input_value("mu_name", "mu_trans")
        self.outputs.xas_data = read_xas_data(
            filename, scan_number, energy_name, mu_name, **retry_options
        )

        super().run()



import logging
from dataclasses import dataclass
from typing import List
from typing import Optional
from typing import Sequence
from typing import Tuple

import h5py
import numpy
from silx.io import h5py_utils

_logger = logging.getLogger(__name__)


@dataclass
class XasMapData:
    mu: numpy.ndarray  # shape (npts, nenergy)
    energy: numpy.ndarray  # shape (npts, nenergy)
    x0: numpy.ndarray  # shape (npts,)
    x1: numpy.ndarray  # shape (npts,)
    x0_name: str
    x1_name: str
    energy_units: Optional[str] = None
    x0_units: Optional[str] = None
    x1_units: Optional[str] = None

    def is_empty(self) -> bool:
        return (
            self.mu.size == 0
            or self.energy.size == 0
            or self.x0.size == 0
            or self.x1.size == 0
        )


@dataclass
class _AccumulateData:
    mu: List[numpy.ndarray]  # npts x shape (nenergy,)
    energy: List[numpy.ndarray]  # npts x shape (nenergy,)
    x0: List[numpy.ndarray]  # shape (npts,)
    x1: List[numpy.ndarray]  # shape (npts,)
    energy_units: Optional[str] = None
    x0_units: Optional[str] = None
    x1_units: Optional[str] = None

    def extend(self, other: "_AccumulateData") -> None:
        self.mu.extend(other.mu)
        self.energy.extend(other.energy)
        self.x0.extend(other.x0)
        self.x1.extend(other.x1)
        self.energy_units = self.energy_units or other.energy_units
        self.x0_units = self.x0_units or other.x0_units
        self.x1_units = self.x1_units or other.x1_units


def read_xasmap(
    filenames: Sequence[str],
    dim0_counter: str,
    dim1_counter: str,
    energy_counter: str,
    mu_counter: str,
    scan_ranges: Optional[Sequence[Optional[Tuple[int, int]]]] = None,
    exclude_scans: Optional[Sequence[Optional[Sequence[int]]]] = None,
    subscan: int = 1,
) -> XasMapData:
    xasmapdata = _AccumulateData(
        mu=list(),
        energy=list(),
        x0=list(),
        x1=list(),
    )

    if not scan_ranges:
        scan_ranges = [None] * len(filenames)
    if not exclude_scans:
        exclude_scans = [None] * len(filenames)

    for filename, scan_range, exclude_scans in zip(
        filenames, scan_ranges, exclude_scans
    ):
        file_xasmapdata = _read_xasmap_file(
            filename,
            dim0_counter,
            dim1_counter,
            energy_counter,
            mu_counter,
            scan_range=scan_range,
            exclude_scans=exclude_scans,
            subscan=subscan,
        )
        xasmapdata.extend(file_xasmapdata)

    return XasMapData(
        mu=numpy.array(xasmapdata.mu),
        energy=numpy.array(xasmapdata.energy),
        x0=numpy.array(xasmapdata.x0),
        x1=numpy.array(xasmapdata.x1),
        x0_name=dim0_counter,
        x1_name=dim1_counter,
        energy_units=xasmapdata.energy_units,
        x0_units=xasmapdata.x0_units,
        x1_units=xasmapdata.x1_units,
    )


def _read_xasmap_file(
    filename: str,
    dim0_counter: str,
    dim1_counter: str,
    energy_counter: str,
    mu_counter: str,
    scan_range: Optional[Tuple[int, int]] = None,
    exclude_scans: Optional[Sequence[int]] = None,
    subscan: int = 1,
) -> _AccumulateData:
    with h5py_utils.File(filename) as nxroot:
        scans = sorted(int(scan.split(".")[0]) for scan in nxroot)
        if scan_range:
            scans = [nr for nr in scans if nr >= scan_range[0] and nr <= scan_range[1]]
        if exclude_scans:
            scans = [nr for nr in scans if nr not in exclude_scans]

        xasmapdata = _AccumulateData(
            mu=list(),
            energy=list(),
            x0=list(),
            x1=list(),
        )

        for scan in scans:
            uri = f"{filename}::/{scan}.{subscan}"
            try:
                scan_xasmapdata = _read_xas_scan(
                    nxroot,
                    scan,
                    subscan,
                    uri,
                    dim0_counter,
                    dim1_counter,
                    energy_counter,
                    mu_counter,
                )
                xasmapdata.extend(scan_xasmapdata)
            except Exception:
                _logger.exception("Error in scan %r", uri)

    return xasmapdata


def _read_xas_scan(
    nxroot: h5py.Group,
    scan: int,
    subscan: int,
    uri: str,
    dim0_counter: str,
    dim1_counter: str,
    energy_counter: str,
    mu_counter: str,
) -> _AccumulateData:
    xasmapdata = _AccumulateData(
        mu=list(),
        energy=list(),
        x0=list(),
        x1=list(),
    )

    # Scan group (Nxentry)
    entry_name = f"{scan}.{subscan}"
    if entry_name not in nxroot:
        _logger.warning("BLISS scan does not exist (%r)", uri)
        return xasmapdata
    nxentry = nxroot[entry_name]
    end_reason = _get_scan_end_reason(nxentry)

    # Measurement group (counters)
    if "measurement" not in nxentry:
        if end_reason:
            _logger.warning("Skip BLISS scan %r (%s)", uri, end_reason)
        else:
            _logger.warning("BLISS scan has no 'measurement' group (%r)", uri)
        return xasmapdata
    measurement = nxentry["measurement"]

    # mu vs. energy
    if energy_counter not in measurement:
        _logger.warning("BLISS scan has no '%s' counter (%r)", energy_counter, uri)
        return xasmapdata
    if mu_counter not in measurement:
        _logger.warning("BLISS scan has no '%s' counter (%r)", mu_counter, uri)
        return xasmapdata
    energy_dset = measurement[energy_counter]
    mu_dset = measurement[mu_counter]

    # Map axes
    x0_dset = None
    x1_dset = None
    if dim0_counter in measurement and dim1_counter in measurement:
        # Bug in ID24-ED dmap: 0D in positioners instead of 1D
        x0_dset = measurement[dim0_counter]
        x1_dset = measurement[dim1_counter]
    elif "instrument/positioners" in nxentry:
        positioners = nxentry["instrument/positioners"]
        if dim0_counter in positioners and dim1_counter in positioners:
            x0_dset = positioners[dim0_counter]
            x1_dset = positioners[dim1_counter]
    if x0_dset is None:
        _logger.warning(
            "BLISS scan has no '%s' or '%s' counter (%r)",
            dim0_counter,
            dim1_counter,
            uri,
        )
        return xasmapdata

    # Shapes (npts, nenergy) or (npts,) or (nenergy,)
    x0_data = _extract_data(x0_dset)
    x1_data = _extract_data(x1_dset)
    energy_data = _extract_data(energy_dset)
    mu_data = _extract_data(mu_dset)

    # Shape (npts,)
    if x0_data.ndim != 1:
        raise ValueError(f"{dim0_counter!r} must be 1D ({uri})")
    if x1_data.ndim != 1:
        raise ValueError(f"{dim1_counter!r} must be 1D ({uri})")

    if mu_data.ndim == 1:
        # One scan == one XAS spectrum
        if end_reason and end_reason != "SUCCESS":
            _logger.warning("Skip single XAS scan %r (%s)", uri, end_reason)
            return xasmapdata

        if energy_data.ndim != 1:
            raise ValueError(f"{energy_counter!r} must be 1D ({uri})")
        if x0_data.size != 1:
            raise ValueError(f"{dim0_counter!r} must be 0D ({uri})")
        if x1_data.size != 1:
            raise ValueError(f"{dim1_counter!r} must be 0D ({uri})")
        xasmapdata = _parse_single_xas_data(
            energy_data,
            mu_data,
            x0_data,
            x1_data,
        )
    else:
        # One scan == many XAS spectra
        xasmapdata = _parse_multi_xas_data(
            energy_data,
            mu_data,
            x0_data,
            x1_data,
        )

    xasmapdata.energy_units = energy_dset.attrs.get("units")
    xasmapdata.x0_units = x0_dset.attrs.get("units")
    xasmapdata.x1_units = x1_dset.attrs.get("units")

    return xasmapdata


def _get_scan_end_reason(nxentry: h5py.Group) -> Optional[str]:
    if "end_reason" not in nxentry:
        return
    end_reason = nxentry["end_reason"][()]
    if isinstance(end_reason, bytes):
        end_reason = end_reason.decode()
    return end_reason


def _parse_single_xas_data(
    energy_data: numpy.ndarray,
    mu_data: numpy.ndarray,
    x0_data: numpy.ndarray,
    x1_data: numpy.ndarray,
) -> _AccumulateData:
    nenergy = min(len(energy_data), len(mu_data))
    x0_data = x0_data[0]
    x1_data = x1_data[0]
    return _AccumulateData(
        mu=[mu_data[:nenergy]],
        energy=[energy_data[:nenergy]],
        x0=[x0_data],
        x1=[x1_data],
    )


def _parse_multi_xas_data(
    energy_data: numpy.ndarray,
    mu_data: numpy.ndarray,
    x0_data: numpy.ndarray,
    x1_data: numpy.ndarray,
) -> _AccumulateData:
    npts, nenergy = mu_data.shape
    if energy_data.ndim == 1:
        # Shape (nenergy,) -> (npts, nenergy)
        energy_data = numpy.tile(energy_data, (npts, 1))
    nenergy = min(nenergy, energy_data.shape[1])
    npts = min(len(mu_data), len(energy_data), len(x0_data), len(x1_data))
    return _AccumulateData(
        mu=mu_data[:npts, :nenergy].tolist(),
        energy=energy_data[:npts, :nenergy].tolist(),
        x0=x0_data[:npts].tolist(),
        x1=x1_data[:npts].tolist(),
    )


def _extract_data(dataset: h5py.Dataset) -> numpy.ndarray:
    """The shape to the returned dataset is 1D or 2D."""
    if dataset.ndim == 0:
        return numpy.array([dataset[()]])
    elif dataset.ndim == 1:
        return dataset[()]
    elif dataset.ndim == 2:
        if dataset.shape[0] == 1:
            return dataset[0]
        else:
            return dataset[()]
    else:
        raise ValueError(
            f"Dataset has {dataset.ndim} dimensions which is not supported ({dataset.file.filename}::{dataset.name})"
        )



import numpy

from ..io.temperature import read_temperature_data
from ..math import planck


def test_fit_temperature_data(xanes_filename):
    temp_data = read_temperature_data(xanes_filename, 39, 2, "up", "T_US")
    temperature0 = temp_data.planck_temperature.copy()
    planck.fit_temperature_data(temp_data)
    temperature1 = temp_data.planck_temperature.copy()

    # Test: changed but close
    numpy.testing.assert_raises(
        AssertionError, numpy.testing.assert_array_equal, temperature0, temperature1
    )
    numpy.testing.assert_allclose(temperature0, temperature1, atol=1e-3)



from dataclasses import dataclass

import numpy

from .bliss import read_counters


@dataclass
class TemperatureData:
    filename: str
    scan_number: int
    label: str
    epoch: numpy.ndarray  # shape: (nPoints,)
    wavelength: numpy.ndarray  # units: nm, shape: (nPoints, nWavelength)
    response: numpy.ndarray  # shape: (nPoints, nWavelength)
    planck_data: numpy.ndarray  # shape: (nPoints, nWavelength)
    planck_fit: numpy.ndarray  # shape: (nPoints, nWavelength)
    planck_fit_slice: slice
    planck_temperature: numpy.ndarray  # units:K, shape: (nPoints,)


def read_temperature_data(
    filename: str,
    scan_number: int,
    subscan_number: int,
    laser_id: str,
    label: str,
    **retry_options,
) -> TemperatureData:
    laser_prefix = "laser_heating"
    laser_counters = [
        "spectrum_lambdas",
        "max_data",
        "planck_data",
        "planck_fit",
        "T_planck",
    ]
    counters = ["epoch"] + [
        f"{laser_prefix}_{laser_id}_{ctr}" for ctr in laser_counters
    ]
    keys = [
        "epoch",
        "wavelength",
        "response",
        "planck_data",
        "planck_fit",
        "planck_temperature",
    ]
    data = read_counters(
        filename, scan_number, subscan_number, counters, **retry_options
    )
    data = dict(zip(keys, data))

    not_zero = numpy.any(numpy.abs(data["planck_fit"]) > 1, axis=0)
    fit_indices = numpy.where(not_zero)[0]
    data["planck_fit_slice"] = slice(fit_indices[0], fit_indices[-1] + 1)

    return TemperatureData(
        filename=filename, scan_number=scan_number, label=label, **data
    )



from ewoksid24.io.temperature import read_temperature_data
from ewoksid24.io.xas import read_xas_data


def test_read_temperature_data(xanes_filename):
    temp_data = read_temperature_data(xanes_filename, 39, 2, "up", "T_US")
    assert temp_data.filename == xanes_filename
    assert temp_data.scan_number == 39

    temp_data = read_temperature_data(xanes_filename, 39, 2, "down", "T_DS")
    assert temp_data.filename == xanes_filename
    assert temp_data.scan_number == 39


def test_read_xas_data(xanes_filename):
    xas_data = read_xas_data(xanes_filename, 39, "energy_enc", "mu_trans")
    assert xas_data.filename == xanes_filename
    assert xas_data.scan_number == 39
